#!/usr/bin/env python

#  
#  Copyright (C) 2009 Jason Smith, Seif Lotfy, Robert Dyer
#                2010 Rico Tzschichholz
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import dbus
import dbus.glib
import sys
import urllib
import os
import subprocess
import gio
import threading

home = "file://" + os.path.expanduser("~")

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from zeitgeist.client import ZeitgeistClient
	from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, StorageState, TimeRange
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

try:
	CLIENT = ZeitgeistClient()
	version = [int(x) for x in CLIENT.get_version()]
	MIN_VERSION = [0, 4, 0, 0]
	if version < MIN_VERSION:
		print "PLEASE USE ZEITGEIST 0.4.0 or above"
		exit()
		

except RuntimeError, e:
	print "Unable to connect to Zeitgeist, won't send events. Reason: '%s'" %e
	exit()
	
class ZGItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.monitor = None 
		self.recent = []
		self.most = []
		self.ids = {}

		self.has_uri = True
		self.uri = self.iface.Get(DOCKITEM_IFACE, "Uri", dbus_interface="org.freedesktop.DBus.Properties")
		if not self.uri:
			self.has_uri = False
			self.uri = self.iface.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties")
			self.uri = 'application://%s' % self.uri[self.uri.rfind('/')+1:]

		self.app_info = None
		app_uri = self.iface.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties")
		apps = gio.app_info_get_all()
		app = None
		t_app = app_uri.split("/")[-1].replace(".desktop", "")
		if t_app:
			for app in apps:
				if t_app in app.get_executable():
					self.app_info = app
					break
		self.update_entries()
	
	def _get_base_template(self):
		base_template = Event()
		if self.has_uri:
			subj = Subject()
			subj.set_uri(self.uri+"/*")
			base_template.set_subjects([subj])
		else:
			base_template.set_actor(self.uri)
		return base_template
	
	def dispose(self):
		if self.monitor:
			CLIENT.remove_monitor(self.monitor)
		DockManagerItem.dispose(self)
	
	def update_entries(self, x=None, y=None, z=None):
		self.recent = []
		self.most = []

		if not self.uri:
			return
		
		def reformat_strings():
			titles = {}
			for r in self.recent:
				if not r[1] in titles:
					titles[r[1]] = []
				titles[r[1]].append(r)
			for m in self.most:
				if not m[1] in titles:
					titles[m[1]] = []
				titles[m[1]].append(m)
			for t in titles:
				if len(titles[t]) > 1:
					for l in titles[t]:
						for i, r in enumerate(self.recent):	
							if l == r:
								self.recent[i][1] = r[1] + " (" + r[0].replace(home, "~") + ")"
								break
						for i, r in enumerate(self.most):	
							if l == r:
								self.most[i][1] = r[1] + " (" + r[0].replace(home, "~") + ")"
								break
						
		def exists(uri):
		 	return uri.startswith("note://") or os.path.exists(urllib.unquote(str(uri[7:])))
		 
		def handle_most(results):
			self.most = []
			for event in results:
				if exists(event.subjects[0].uri):
					self.most.append([event.subjects[0].uri, event.subjects[0].text])
				if len(self.most) >= 5:
					break
			reformat_strings()
			self.update_menu()
			
			if not self.monitor:
				self.monitor = CLIENT.install_monitor(TimeRange.always(), [template], self.update_entries, self.update_entries)
				
		def handle_recent(results):
			self.recent = []
			for event in results:
				if exists(event.subjects[0].uri):
					self.recent.append([event.subjects[0].uri, event.subjects[0].text])
					subj = Subject()
					subj.uri = "!"+event.subjects[0].uri
					template.subjects.append(subj)
				if len(self.recent) >= 5:
					break
			CLIENT.find_events_for_templates([template], handle_most, num_events = 1000, result_type = 4)
		 
		template = self._get_base_template()
		CLIENT.find_events_for_templates([template], handle_recent, num_events = 1000, result_type = 2)


	def update_menu(self):
		for id in self.ids.keys():
			self.remove_menu_item(id)
		self.ids.clear()
		uris = self.most
		for uri in uris:
			uri, title = uri[0], uri[1]
			icon = self.get_icon(uri)
			self.ids[(self.add_menu_item(title, icon, "Other Popular"))] = uri
		uris = self.recent
		for uri in uris:
			uri, title = uri[0], uri[1]
			icon = self.get_icon(uri)
			self.ids[(self.add_menu_item(title, icon, "Latest"))] = uri
	
	def get_icon(self, uri):
		if uri.startswith("note://"):
			return "tomboy-note"
 		file_ = gio.File(uri)
		info = file_.query_info('standard::icon,thumbnail::path')
		thumbnail_path = info.get_attribute_as_string('thumbnail::path')
		if thumbnail_path:
			return thumbnail_path
		else:
		 	icon = info.get_icon() 
		 	icon_path = None
		 	if isinstance(icon, gio.ThemedIcon): 
				icon_path  = icon.get_names()[0]
				#FIXME Handle the list of icons 
				# Docky supports a list of icons separated by ";;"
				#icon_path  = ";;".join(icon.get_names())
			elif isinstance(icon, gio.FileIcon): 
				icon_path = icon.get_file().get_path()
			return icon_path
		
	def menu_pressed(self, menu_id):
		if menu_id in self.ids.keys():
			if self.app_info:
				self.app_info.launch([gio.File(self.ids[menu_id])])	
			else:
				subprocess.Popen(['xdg-open', self.ids[menu_id]])

class ZGSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "Uri", dbus_interface="org.freedesktop.DBus.Properties") != "" or item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties") != "":
			self.items[pathtoitem] = ZGItem(self, pathtoitem)

zgsink = ZGSink()

def cleanup ():
	zgsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)
	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))
	while mainloop.is_running():
		mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2009-2010 Jason Smith, Rico Tzschichholz, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import glib
import dbus
import dbus.glib
import sys
import os

try:
	import gtk
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from dockmanager.dockmanager import RESOURCESDIR
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	print e
	exit()

bansheebus = "org.bansheeproject.Banshee"
playerpath = "/org/bansheeproject/Banshee/PlayerEngine"
playeriface = "org.bansheeproject.Banshee.PlayerEngine"

controlpath = "/org/bansheeproject/Banshee/PlaybackController"
controliface = "org.bansheeproject.Banshee.PlaybackController"

enable_art_icon = True;
enable_badge_text = True;

album_art_tmpfile = "/tmp/dockmanager_%s_banshee_helper.png" % os.getenv('USERNAME')

# 0 - none, 1- jewel, 2 - vinyl)
overlay = 2

class BansheeItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.timer = 0
		self.player = None
		self.control = None
		
		self.duration_secs = 0
		self.songinfo = None
		self.current_arturl = ""
		self.current_arturl_mtime = 0
				
		self.bus.add_signal_receiver(self.name_owner_changed_cb,
                                             dbus_interface='org.freedesktop.DBus',
                                             signal_name='NameOwnerChanged')
                                             
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		
		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)
	
		self.bus.add_signal_receiver(self.signal_EventChanged, "EventChanged",  playeriface, bansheebus, playerpath)
		self.bus.add_signal_receiver(self.signal_StateChanged, "StateChanged",  playeriface, bansheebus, playerpath)

	def list_names_handler(self, names):
		if bansheebus in names:
			self.init_banshee_objects()
			self.set_menu_buttons()
			self.update_text()			
			self.update_badge()
			self.update_icon()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == bansheebus:
			if new_owner:
				self.init_banshee_objects()
			else:
				self.player = None
				self.control = None
				self.stop_timer()
				self.set_menu_buttons()
				self.update_text()			
				self.update_badge()
				self.update_icon()
	
	def start_timer(self):
		if not self.timer > 0:
			self.timer = gobject.timeout_add (1000, self.update_badge)
	
	def stop_timer(self):
		if self.timer > 0:
			gobject.source_remove (self.timer)
			self.timer = 0
	
	def init_banshee_objects(self):
		obj = self.bus.get_object(bansheebus, playerpath)
		self.player = dbus.Interface(obj, playeriface)
		
		obj = self.bus.get_object(bansheebus, controlpath)
		self.control = dbus.Interface(obj, controliface)
		
		if self.player:
			self.update_songinfo()
		
		if self.banshee_is_playing():
			self.start_timer()

	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
		
		if not self.player:
			return

		self.add_menu_item("Previous", "media-skip-backward")
		if self.banshee_is_playing():
			self.add_menu_item("Pause", "media-playback-pause")
		else:
			self.add_menu_item("Play", "media-playback-start")
		self.add_menu_item("Next", "media-skip-forward")
	
	def signal_EventChanged(self, event, st, value):
		print "EventChanged: %s" % (event)	
		if (event in ["startofstream", "trackinfoupdated"]):
			self.update_songinfo();
			self.update_text()
			self.update_badge()
			self.update_icon()
		elif (event in ["endofstream"]):
			self.update_icon()
			self.update_text()
		elif (event in ["seek"]):
			self.update_badge()

	def signal_StateChanged(self, state):
		print "StateChanged: %s" % (state)	
		self.set_menu_buttons()
		self.update_text()

		if state == "playing":
			self.start_timer()
		else:
			self.stop_timer()
			
	def update_songinfo(self):
		if self.player:
			try:
				song = self.player.GetCurrentTrack()
				self.duration_secs = self.player.GetLength() / 1000
				if self.duration_secs > 0:
					self.songinfo = '%s - %s (%i:%02i)' % (song.get("artist", "Unknown"), song.get("name", "Unknown"), self.duration_secs / 60, self.duration_secs % 60)
				else:
					self.songinfo = '%s - %s' % (song.get("artist", "Unknown"), song.get("name", "Unknown"))
			except dbus.DBusException, e:
				self.duration_secs = 0
				self.songinfo = None
			return
		self.duration_secs = 0
		self.songinfo = None
	
	def update_icon(self):
		if not self.player:
			self.current_arturl = ""
			self.reset_icon()
			return False
			
		if not enable_art_icon:
			return True
		
		if self.banshee_is_playing():
			arturl = self.get_album_art_path()
			# Add overlay to cover
			if os.path.isfile(arturl):
				if self.current_arturl == arturl and self.current_arturl_mtime == os.stat(arturl).st_mtime:
					return True
				self.current_arturl = arturl
				self.current_arturl_mtime = os.stat(arturl).st_mtime
				self.set_icon(self.get_album_art_overlay_path(arturl))
			else:
				self.current_arturl = ""
				self.reset_icon()
		else:
			self.current_arturl = ""
			self.reset_icon()
		return True
		
	def get_album_art_path(self):
		artwork_id = self.player.GetCurrentTrack().get("artwork-id")

		if not self.player or not artwork_id:
			return ""

		arturl = os.path.expanduser("~/.cache/media-art/%s.jpg" % artwork_id)
		if not os.path.isfile(arturl):
			arturl = os.path.expanduser("~/.cache/album-art/%s.jpg" % artwork_id)

		return arturl

	def get_album_art_overlay_path(self, picfile):
		if overlay == 0:
			return picfile

		try:
			pb = gtk.gdk.pixbuf_new_from_file(picfile)
		except Exception, e:
			print e
			return picfile
		
		pb_result = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 250, 250)
		pb_result.fill(0x00000000)

		if overlay == 1:
			overlayfile = os.path.join(RESOURCESDIR, "albumoverlay_jewel.png")
			pb.composite(pb_result, 30, 21, 200, 200, 30, 21, 200.0/pb.get_width(), 200.0/pb.get_height(), gtk.gdk.INTERP_BILINEAR, 255)
		elif overlay == 2:
			overlayfile = os.path.join(RESOURCESDIR, "albumoverlay_vinyl.png")
			pb.composite(pb_result, 3, 26, 190, 190, 3, 26, 190.0/pb.get_width(), 190.0/pb.get_height(), gtk.gdk.INTERP_BILINEAR, 255)
		else:
			return picfile

		pb_overlay = gtk.gdk.pixbuf_new_from_file_at_size(overlayfile, 250, 250)
		pb_overlay.composite(pb_result, 0, 0, 250, 250, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
		pb_result.save(album_art_tmpfile, "png", {})
		
		return album_art_tmpfile

	def update_text(self):
		if not self.player:
			self.reset_tooltip()

		if self.banshee_is_playing() and self.songinfo:
			self.set_tooltip(self.songinfo)
		else:
			self.reset_tooltip()
	
	def update_badge(self):
		if not self.player:
			self.reset_badge()
			return False

		if not enable_badge_text:
			return True
		
		if self.banshee_is_playing():
			#if song length is 0 then counting up instead of down
			if self.duration_secs > 0:
				position = self.duration_secs - self.player.GetPosition() / 1000
			else:
				position = self.player.GetPosition() / 1000
			string = '%i:%02i' % (position / 60, position % 60)
			self.set_badge(string)
		else:
			self.reset_badge()
		return True
	
	def menu_pressed(self, menu_id):
		if not self.player:
			return False
		
		if self.id_map[menu_id] == "Play":
			self.player.Play()
		elif self.id_map[menu_id] == "Pause":
			self.player.Pause()
		elif self.id_map[menu_id] == "Next":
			self.control.Next(False)
		elif self.id_map[menu_id] == "Previous":
			self.control.Previous(False)
		
	def banshee_is_playing(self):
		if self.player:
			try:
				return self.player.GetCurrentState() == "playing"
			except dbus.DBusException, e:
				return False
		return False
	

class BansheeSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("banshee-1.desktop"):
			self.items[pathtoitem] = BansheeItem(self, pathtoitem)

bansheesink = BansheeSink()

def cleanup ():
	bansheesink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	while mainloop.is_running():
		mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Erigami Scholey-Fuller
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


import atexit
import gobject
import glib
import sys
import os
import time
import dbus
import math

try:
    from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
    from signal import signal, SIGTERM
    from sys import exit
except ImportError, e:
    print `e`
    exit()

break_names = {
    'micro_pause' : "Micro break",
    'rest_break' : 'Rest break',
    'daily_limit' : 'Daily limit'
}

class Workrave:
    """Interface to Workrave."""

    def connect(self, bus):
        obj = bus.get_object("org.workrave.Workrave",
                "/org/workrave/Workrave/Core")
        self.workrave = dbus.Interface(obj, "org.workrave.CoreInterface")
        self.config = dbus.Interface(obj, "org.workrave.ConfigInterface")


    def findNextEnd(self):
        """Find the next timer to go off.

        Returns an array of tuples. Each typle consists of 
        (seconds-until-timer-ends, length-of-timer, time-timer-ends, name-of-timer), 
        where the timer is the next timer scheduled to end.

        If there are no ticking timers, an empty array is returned. 
        """
        # Pick a really large number for the next timer
        remaining = 2 ** 18

        toReturn = []
        for (timerName, configName) in [['microbreak', 'micro_pause'], ['restbreak', 'rest_break'], ['dailylimit', 'daily_limit']]:
            if not self.workrave.IsTimerRunning(timerName):
                continue
            (length, found) = self.config.GetInt('timers/%s/limit' % configName)
            if not found:
                continue

            elapsed = self.workrave.GetTimerElapsed(timerName)
            r = length - elapsed

            tuple = (r, length, r + time.time(), configName)
            toReturn.append(tuple)

        def c(a, b):
            return cmp(a[0], b[0])

        toReturn.sort(c)

        return toReturn


class WorkraveWatcher:
    """Responsible for monitoring Workrave and updating the UI."""

    def __init__(self):
        self.wr = Workrave()
        self.dock_item = None
        self.workrave_state = None
        self.breaks = []
        self.timers = []


    def on_service_change(self, bus, conn):
        if "" == conn:
            # Workrave has gone away
            self.workrave_state = None
            self.breaks = []
            
            for timer in self.timers:
                gobject.source_remove(timer)

            self.timers = []

            return

        # A new workrave is here to play with us
        try:
            self.wr.connect(bus)
        except dbus.exceptions.DBusException, e:
            return

        self.wr.workrave.connect_to_signal('MicrobreakChanged', 
                lambda x: self.__state_change('micro', x))
        self.wr.workrave.connect_to_signal('RestbreakChanged', 
                lambda x: self.__state_change('rest', x))
        self.wr.workrave.connect_to_signal('DailylimitChanged', 
                lambda x: self.__state_change('daily', x))

        self.timers.append(gobject.timeout_add(5000, self.__poll_workrave))
        self.timers.append(gobject.timeout_add(1000, self.__update_ui))


    def __state_change(self, timer, progress):
        self.workrave_state = progress
        self.__poll_workrave()


    def __poll_workrave(self):
        self.breaks = self.wr.findNextEnd()

        return True

    def __update_ui(self):
        message = ""
        progress = 0
        if self.workrave_state == 'break':
            message = 'Break'
        else:
            next_break = 0
            duration = 0
            if len(self.breaks) > 0:
                next_break = self.breaks[0][2]
                duration = self.breaks[0][1]

            if next_break <= 0:
                message = "(pause)"
            else:
                rem = next_break - time.time()
                message = self.__humanize_time(rem)
                if rem < 0:
                    progress = 100
                else:
                    progress = 100 - int(100 * (float(rem) / float(duration)))

        if self.dock_item:
            self.dock_item.set_message(message)
            self.dock_item.set_progress(progress)
            self.dock_item.set_tooltip(self.__calculate_tooltip())

        return True

    def __calculate_tooltip(self):
        toReturn = ''

        for timer in self.breaks:
            if len(toReturn) > 0:
                toReturn = toReturn + ",  "

            secondsLeft = int(timer[2] - time.time())
            msg = self.__humanize_time(secondsLeft, False)
            toReturn = toReturn + ("%s in %s" % (break_names[timer[3]], msg))

        if len(toReturn) == 0:
            toReturn = 'Workrave'

        toReturn = toReturn + "  "

        return toReturn
            

    def __humanize_time(self, rem, hide_seconds=True):
        if rem < 0:
            minutes = -1 * math.ceil(float(rem) / 60)
            seconds = (-1 * rem) % 60
            message = "-%i:%02i" % (minutes, seconds)
        else:
            minutes = int(rem / 60)
            seconds = rem % 60
            if hide_seconds and minutes >= 5:
                message = "%im" % minutes
            else:
                message = "%i:%02i" % (minutes, seconds)
        
        return message
        
    def register_dock_item(self, item):
        self.dock_item = item


    def dispose(self):
        if self.dock_item:
            self.dock_item.set_message("")


    def set_item(self, item):
        """Called to set the item we're showing in the dock"""
        self.dock_item = item

watcher = WorkraveWatcher()


class WorkraveItem(DockManagerItem):

    def __init__(self, sink, path):
        DockManagerItem.__init__(self, sink, path)

        # Initialize our dbus interface
        self.__dbus = dbus.SessionBus()

        watcher.set_item(self)
        #self.add_menu_item("Preferences", "preferences", "Preferences")

    def set_message(self, msg):
        self.iface.UpdateDockItem({"message": msg})

    def menu_pressed(self, menu_id):
        pass


class WorkraveSink(DockManagerSink):
    def item_path_found(self, pathtoitem, item):
        path = item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties")
        if path.endswith("workrave.desktop"):
            self.items[pathtoitem] = WorkraveItem(self, pathtoitem)
            watcher.register_dock_item(self.items[pathtoitem])
                         

def cleanup():
    watcher.dispose()
    sink.dispose()

sink = WorkraveSink()


if __name__ == "__main__":
    bus = dbus.SessionBus()
    bus.watch_name_owner("org.workrave.Workrave", lambda x: watcher.on_service_change(bus, x))

    mainloop = gobject.MainLoop(is_running=True)
     
    atexit.register(cleanup)
    signal(SIGTERM, lambda signum, stack_frame: exit(1))

    mainloop.run()



#!/usr/bin/env python
# -.- coding: utf-8 -.-

#
# Copyright (c) 2009 Seif Lotfy, Siegfried Gevatter
# Copyright (c) 2010 Robert Dyer
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import gobject
import sys
import urllib
import datetime
import time
import os
import gtk
import gnome.ui
import atexit

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from zeitgeist.client import ZeitgeistClient
	from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, StorageState
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

try:
	CLIENT = ZeitgeistClient()
	version = [int(x) for x in CLIENT.get_version()]
	MIN_VERSION = [0, 3, 1, 0]
	if version < MIN_VERSION:
		print "PLEASE USE ZEITGEIST 0.3.1 or above"
		exit()

except RuntimeError, e:
	print "Unable to connect to Zeitgeist, won't send events. Reason: '%s'" %e
	exit()

INTERPRETATION = {
		"http://zeitgeist-project.com/schema/1.0/core#VisitEvent":"OPENED",
		"http://zeitgeist-project.com/schema/1.0/core#ModifyEvent":"SAVED",
		"http://zeitgeist-project.com/schema/1.0/core#CreateEvent":"CREATED"
		}

class LaunchManager:
    """
    A program lauching utility which handles opening a URI or executing a
    program or .desktop launcher, handling variable expansion in the Exec
    string.

    Adds the launched URI or launcher to the ~/.recently-used log. Sets a
    DESKTOP_STARTUP_ID environment variable containing useful information such
    as the URI which caused the program execution and a timestamp.

    See the startup notification spec for more information on
    DESKTOP_STARTUP_IDs.
    """
    
    def __init__(self):
        self.recent_model = None

    def _get_recent_model(self):
        # FIXME: This avoids import cycles
        if not self.recent_model:
            import zeitgeist_recent
            self.recent_model = zeitgeist_recent.recent_model
        return self.recent_model
    
    def launch_uri(self, uri, mimetype = None):
        assert uri, "Must specify URI to launch"
    
        child = os.fork()
        if not child:
            # Inside forked child
            os.setsid()
            os.environ['zeitgeist_LAUNCHER'] = uri
            os.environ['DESKTOP_STARTUP_ID'] = self.make_startup_id(uri)
            os.spawnlp(os.P_NOWAIT, "gnome-open", "gnome-open", uri)
            os._exit(0)
        else:
            os.wait()
            if not mimetype:
                mimetype = "application/octet-stream"
            try:
                # Use XDG to lookup mime type based on file name.
                # gtk_recent_manager_add_full requires it.
                import xdg.Mime
                mimetype = xdg.Mime.get_type_by_name(uri)
                if mimetype:
                    mimetype = str(mimetype)
                return mimetype
            except (ImportError, NameError):
                # No mimetype found for URI: %s
                pass
        return child
    
    def get_local_path(self, uri):
        scheme, path = urllib.splittype(uri)
        if scheme == None:
            return uri
        elif scheme == "file":
            path = urllib.url2pathname(path)
            if path[:3] == "///":
                path = path[2:]
            return path
        return None
    
    def launch_command_with_uris(self, command, uri_list, launcher_uri = None):
        if command.rfind("%U") > -1:
            uri_str = ""
            for uri in uri_list:
                uri_str = uri_str + " " + uri
                return self.launch_command(command.replace("%U", uri_str), launcher_uri)
        elif command.rfind("%F") > -1:
            file_str = ""
            for uri in uri_list:
                uri = self.get_local_path(self, uri)
                if uri:
                    file_str = file_str + " " + uri
                else:
                    # Command does not support non-file URLs
                    pass
            return self.launch_command(command.replace("%F", file_str), launcher_uri)
        elif command.rfind("%u") > -1:
            startup_ids = []
            for uri in uri_list:
                startup_ids.append(self.launch_command(command.replace("%u", uri), launcher_uri))
            else:
                return self.launch_command(command.replace("%u", ""), launcher_uri)
            return startup_ids
        elif command.rfind("%f") > -1:
            startup_ids = []
            for uri in uri_list:
                uri = self.get_local_path(self, uri)
                if uri:
                    startup_ids.append(self.launch_command(command.replace("%f", uri),
                                                           launcher_uri))
                else:
                    #print " !!! Command does not support non-file URLs: ", command
                    pass
            else:
                return self.launch_command(command.replace("%f", ""), launcher_uri)
            return startup_ids
        else:
            return self.launch_command(command, launcher_uri)
    
    def make_startup_id(self, key, ev_time = None):
        if not ev_time:
            ev_time = gtk.get_current_event_time()
        if not key:
            return "zeitgeist_TIME%d" % ev_time
        else:
            return "zeitgeist:%s_TIME%d" % (key, ev_time)

    def parse_startup_id(self, id):
        if id and id.startswith("zeitgeist:"):
            try:
                uri = id[len("zeitgeist:"):id.rfind("_TIME")]
                timestamp = id[id.rfind("_TIME") + len("_TIME"):]
                return (uri, timestamp)
            except IndexError:
                pass
        return (None, None)

    def launch_command(self, command, launcher_uri = None):
        startup_id = self.make_startup_id(launcher_uri)
        child = os.fork()
        if not child:
            # Inside forked child
            os.setsid()
            os.environ['DESKTOP_STARTUP_ID'] = startup_id
            if launcher_uri:
                os.environ['zeitgeist_LAUNCHER'] = launcher_uri
            os.system(command)
            os._exit(0)
        else:
            os.wait()
           
            return (child, startup_id)

class IconFactory():
    """
    Icon lookup swiss-army knife (from menutreemodel.py)
    """
    
    def __init__(self):
        self.icon_dict={}
    
    def load_icon_from_path(self, icon_path, icon_size=None):
        try:
            if icon_size:
                icon = gtk.gdk.pixbuf_new_from_file_at_size(icon_path,
                    int(icon_size), int(icon_size))
                return icon
            else:
                return gtk.gdk.pixbuf_new_from_file(icon_path)
        except Exception:
            pass
        return None
    
    def load_icon_from_data_dirs(self, icon_value, icon_size=None):
        data_dirs = None
        if os.environ.has_key("XDG_DATA_DIRS"):
            data_dirs = os.environ["XDG_DATA_DIRS"]
        if not data_dirs:
            data_dirs = "/usr/local/share/:/usr/share/"

        for data_dir in data_dirs.split(":"):
            retval = self.load_icon_from_path(os.path.join(data_dir, "pixmaps", icon_value),
                                              icon_size)
            if retval:
                return retval
            
            retval = self.load_icon_from_path(os.path.join(data_dir, "icons", icon_value),
                                              icon_size)
            if retval:
                return retval
        return None
    
    def transparentize(self, pixbuf, percent):
        pixbuf = pixbuf.add_alpha(False, '0', '0', '0')
        for row in pixbuf.get_pixels_array():
            for pix in row:
                pix[3] = min(int(pix[3]), 255 - (percent * 0.01 * 255))
        return pixbuf
    
    def greyscale(self, pixbuf):
        pixbuf = pixbuf.copy()
        for row in pixbuf.get_pixels_array():
            for pix in row:
                pix[0] = pix[1] = pix[2] = (int(pix[0]) + int(pix[1]) + int(pix[2])) / 3
        return pixbuf
    
    def load_icon(self, icon_value, icon_size, force_size=True, cache=True):
        if not self.icon_dict.has_key(str(icon_value)+str(icon_size)) or not cache:
            try:
                if isinstance(icon_value, gtk.gdk.Pixbuf):
                    return icon_value
                elif os.path.isabs(icon_value):
                    icon = self.load_icon_from_path(icon_value, icon_size)
                    if icon:
                        return icon
                    icon_name = os.path.basename(icon_value)
                else:
                    icon_name = icon_value
                
                if icon_name.endswith(".png"):
                    icon_name = icon_name[:-len(".png")]
                elif icon_name.endswith(".xpm"):
                    icon_name = icon_name[:-len(".xpm")]
                elif icon_name.endswith(".svg"):
                    icon_name = icon_name[:-len(".svg")]
                
                icon = None
                info = icon_theme.lookup_icon(icon_name, icon_size, gtk.ICON_LOOKUP_USE_BUILTIN)
                if info:
                    if icon_name.startswith("gtk-"):
                        icon = info.load_icon()
                    elif info.get_filename():
                        icon = self.load_icon_from_path(info.get_filename(), icon_size)
                else:
                    icon = self.load_icon_from_data_dirs(icon_value, icon_size) 
                
                if cache:
                    self.icon_dict[str(icon_value)+str(icon_size)] = icon
                return icon
            except Exception:
                self.icon_dict[str(icon_value)+str(icon_size)] = None
                return None
        else:
            return self.icon_dict[str(icon_value)+str(icon_size)]
    
    def load_image(self, icon_value, icon_size, force_size=True):
        pixbuf = self.load_icon(icon_value, icon_size, force_size)
        img = gtk.Image()
        img.set_from_pixbuf(pixbuf)
        img.show()
        return img

    def make_icon_frame(self, thumb, icon_size = None, blend = False):
        border = 1

        mythumb = gtk.gdk.Pixbuf(thumb.get_colorspace(),
                                 True,
                                 thumb.get_bits_per_sample(),
                                 thumb.get_width(),
                                 thumb.get_height())
        mythumb.fill(0x00000080) # black, 50% transparent
        if blend:
            thumb.composite(mythumb, 0.5, 0.5,
                            thumb.get_width(), thumb.get_height(),
                            0.5, 0.5,
                            0.5, 0.5,
                            gtk.gdk.INTERP_NEAREST,
                            256)
        thumb.copy_area(border, border,
                        thumb.get_width() - (border * 2), thumb.get_height() - (border * 2),
                        mythumb,
                        border, border)
        return mythumb
    
    
class Thumbnailer:
    
    def __init__(self):
        self.icon_dict={}

    def get_icon(self, subject, icon_size, timestamp = 0):
        
        uri = subject.uri
        if not self.icon_dict.get(uri+str(icon_size)):
            cached_icon = self._lookup_or_make_thumb(uri, subject.mimetype,
                icon_size, timestamp)
            self.icon_dict[uri+str(icon_size)] = cached_icon
        
        return self.icon_dict[uri+str(icon_size)]

    def _lookup_or_make_thumb(self, uri, mimetype, icon_size, timestamp):
        icon_name, icon_type = \
                   gnome.ui.icon_lookup(icon_theme, thumb_factory, uri, mimetype, 0)
        try:
            if icon_type == gnome.ui.ICON_LOOKUP_RESULT_FLAGS_THUMBNAIL or \
                   thumb_factory.has_valid_failed_thumbnail(uri,int(timestamp)):
                # Use existing thumbnail
                thumb = icon_factory.load_icon(icon_name, icon_size)
            elif self._is_local_uri(uri):
                # Generate a thumbnail for local files only
                thumb = thumb_factory.generate_thumbnail(uri, mimetype)
                thumb_factory.save_thumbnail(thumb, uri, timestamp)

            if thumb:
                thumb = icon_factory.make_icon_frame(thumb, icon_size)
                return thumb
            
        except Exception:
            pass

        return icon_factory.load_icon(icon_name, icon_size)

    def _is_local_uri(self, uri):
        # NOTE: gnomevfs.URI.is_local seems to hang for some URIs (e.g. ssh
        #        or http).  So look in a list of local schemes which comes
        #        directly from gnome_vfs_uri_is_local_scheme.
        scheme, path = urllib.splittype(self.get_uri() or "")
        return not scheme or scheme in ("file", "help", "ghelp", "gnome-help", "trash",
                                        "man", "info", "hardware", "search", "pipe",
                                        "gnome-trash")

class CellRendererPixbuf(gtk.CellRendererPixbuf):
    
    __gsignals__ = {
        'toggled': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
        (gobject.TYPE_STRING,))
    }
    def __init__(self):
        gtk.CellRendererPixbuf.__init__(self)
        self.set_property('mode', gtk.CELL_RENDERER_MODE_ACTIVATABLE)
    
    def do_activate(self, event, widget, path, background_area, cell_area, flags):
        model = widget.get_model()
        self.emit("toggled",path)

icon_theme = gtk.icon_theme_get_default()
icon_factory = IconFactory()
thumbnailer = Thumbnailer()
thumb_factory = gnome.ui.ThumbnailFactory("normal")
launcher = LaunchManager()


from zeitgeist.datamodel import Event, Subject, Interpretation, Manifestation, StorageState

class Window(gtk.Window):
	def __init__(self, CLIENT):
		gtk.Window.__init__(self)
		self.view = DataIconView()
		scroll = gtk.ScrolledWindow()
		scroll.add_with_viewport(self.view)
		self.add(scroll)
		scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self._zg = CLIENT
		self.set_size_request(600,480)
		self.show_all()

	def load_events(self, start=0, end=0, actor=None):
		self.set_title("Journal for "+actor)
		def exists(uri):
			return not uri.startswith("file://") or os.path.exists(urllib.unquote(str(uri[7:])))

		def _handle_find_events(ids):
			self._zg.get_events(ids, _handle_get_events)

		def _handle_get_events(events):
			uris = []
			for event in events:
				for subject in event.subjects:
					if exists(subject.uri):
						self.view.append_object(event)
		
		event = Event()
		event.set_actor(actor)
		self._zg.find_events_for_templates([event],_handle_get_events, [start, end], StorageState.Any, 0, 4) 


class DataIconView(gtk.TreeView):
	def __init__(self):
		gtk.TreeView.__init__(self)
		self.store = gtk.TreeStore(gtk.gdk.Pixbuf,
					str,	#TIME
					str,	#INTERPRETATION
					str,	#Subject_TEXT
					gobject.TYPE_PYOBJECT
					)

		# Icon 
		icon_cell = gtk.CellRendererPixbuf()
		icon_cell.set_property("yalign", 0.5)
		icon_column = gtk.TreeViewColumn("Icon",icon_cell,pixbuf=0)
		
		time_cell = gtk.CellRendererText()
		time_column = gtk.TreeViewColumn("Time", time_cell, markup=1)
				
		interp_cell = gtk.CellRendererText()
		interp_column = gtk.TreeViewColumn("Interpretation", interp_cell, markup=2)
		
		subject_cell = gtk.CellRendererText()
		subject_column = gtk.TreeViewColumn("Subject", subject_cell, markup=3)
		
		self.append_column(time_column)
		self.append_column(icon_column)
		self.append_column(subject_column)
		self.append_column(interp_column)
		self.set_model(self.store)

		self.set_headers_visible(False)
		self.iter = {"uri":None, "iter":None}

	def clear(self):
		self.store.clear()

	def append_object(self, event):

		icon = thumbnailer.get_icon(event.subjects[0], 32)
		date = int(event.timestamp)/1000
		date = datetime.datetime.fromtimestamp(date).strftime("%H:%M:%S		%A, %D") 

		iterator = None
		if self.iter["uri"] == event.subjects[0].uri:
			self.store.append(self.iter["iter"], 
						[icon,
						date,
						INTERPRETATION[event.interpretation],
						event.subjects[0].text,
						event])
		else:
			iterator = None
			self.iter["uri"] = event.subjects[0].uri
			self.iter["iter"] = self.store.append(None, [icon,
								date,
								INTERPRETATION[event.interpretation],
								event.subjects[0].text,
								event])


class JournalItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		menu_id = self.add_menu_item("Journal", "document-open-recent")
		self.id_map[menu_id] = "Journal"
		self.uri = self.iface.Get(DOCKITEM_IFACE, "Uri", dbus_interface="org.freedesktop.DBus.Properties")
		if not self.uri:
			self.uri = self.iface.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties")
			self.uri = 'application://%s' % self.uri[self.uri.rfind('/')+1:]

	def menu_pressed(self, menu_id):
		if self.id_map[menu_id] == "Journal":
			window = Window(CLIENT)
			window.load_events(0, time.time() * 1000, self.uri)


class JournalSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "Uri", dbus_interface="org.freedesktop.DBus.Properties") or item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties"):
			self.items[pathtoitem] = JournalItem(self, pathtoitem)


zgjournalsink = JournalSink()

def cleanup ():
	zgjournalsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	while mainloop.is_running():
	    mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2009-2010 Jason Smith, Rico Tzschichholz, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import glib
import dbus
import dbus.glib
import sys
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

tomboybus = "org.gnome.Tomboy"
tomboypath = "/org/gnome/Tomboy/RemoteControl"
tomboyiface = "org.gnome.Tomboy.RemoteControl"

class TomboyItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.tomboy = None
		
		self.bus.add_signal_receiver(self.name_owner_changed_cb,
                                             dbus_interface='org.freedesktop.DBus',
                                             signal_name='NameOwnerChanged')
                                             
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		
		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)
	
		self.bus.add_signal_receiver(self.note_added, "NoteAdded", tomboyiface, tomboybus, tomboypath)
		self.bus.add_signal_receiver(self.note_deleted, "NoteDeleted", tomboyiface, tomboybus, tomboypath)
		self.bus.add_signal_receiver(self.note_changed, "NoteChanged", tomboyiface, tomboybus, tomboypath)
	
	def list_names_handler(self, names):
		if tomboybus in names:
			self.init_tomboy_objects()
			self.set_menu_buttons()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == tomboybus:
			if new_owner:
				self.init_tomboy_objects()
			else:
				self.tomboy = None

			self.set_menu_buttons()
	
	def init_tomboy_objects(self):
		obj = self.bus.get_object(tomboybus, tomboypath)
		self.tomboy = dbus.Interface(obj, tomboyiface)
		
	def note_added (self, note):
		self.set_menu_buttons()
		
	def note_deleted (self, note, a):
		self.set_menu_buttons()
	
	def note_changed (self, note):
		self.set_menu_buttons()
	
	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
		
		if not self.tomboy:
			return

		self.add_menu_item ("Create New Note", "tomboy", "Tomboy Controls")
		self.add_menu_item ("Find Note...", "gtk-find", "Tomboy Controls")
		
		for note in self.get_tomboy_menu_notes ():
			self.add_menu_item (self.tomboy.GetNoteTitle(note), "tomboy", "Notes")
		
	def get_tomboy_menu_notes(self):
		notes = self.tomboy.ListAllNotes()
		notes.sort(key=self.tomboy.GetNoteChangeDate)
		notes.reverse()
		return notes[:9]
		
	def menu_pressed(self, menu_id):
		if not menu_id in self.id_map:
			return	
		
		menu_id = self.id_map[menu_id]
		
		if menu_id == "Create New Note":
			newnote = self.tomboy.CreateNote()
			self.tomboy.DisplayNote(newnote)
		elif menu_id == "Find Note...":
			self.tomboy.DisplaySearch()
		else:
			for note in self.get_tomboy_menu_notes ():
				if menu_id == self.tomboy.GetNoteTitle(note):
					self.tomboy.DisplayNote(note)
		
class TomboySink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("tomboy.desktop"):
			self.items[pathtoitem] = TomboyItem(self, pathtoitem)

tomboysink = TomboySink()

def cleanup ():
	tomboysink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Lukasz Marcinowski, Rico Tzschichholz
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import dbus
import dbus.glib
import glib
import sys
import os
import time
import dbus.service

import time

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink
	from dockmanager.dockmanager import RESOURCESDIR, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

skypebus = "com.Skype.API"
skypepath = "/com/Skype"
skypeclientpath = "/com/Skype/Client"
skypeclientitem = "com.Skype.API.Client"
skypeitem = "com.Skype.API"

STATUS_ICON_OFFLINE = os.path.join(RESOURCESDIR, "skype_offline.svg")
STATUS_ICON_ONLINE = os.path.join(RESOURCESDIR, "skype_online.svg")
STATUS_ICON_AWAY = os.path.join(RESOURCESDIR, "skype_away.svg")
STATUS_ICON_SKYPEME = os.path.join(RESOURCESDIR, "skype_skypeme.svg")
STATUS_ICON_DND = os.path.join(RESOURCESDIR, "skype_dnd.svg")
STATUS_ICON_NA = os.path.join(RESOURCESDIR, "skype_na.svg")
STATUS_ICON_INVISIBLE = os.path.join(RESOURCESDIR, "skype_invisible.svg")


class SkypeNotify(dbus.service.Object):
	def __init__(self, bus, notify):
		dbus.service.Object.__init__(self, dbus.SessionBus(), skypeclientpath)
		self.notify = notify
	
	@dbus.service.method(dbus_interface=skypeclientitem)	
	def Notify(self, com):
		self.notify(unicode(com))

class DockySkypeItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.items_unread = 0
		self.skype = None
		self.skype_in = SkypeNotify(self.bus, self.notify_cb)

		self.owner_changed_signal = self.bus.add_signal_receiver(
					handler_function = self.name_owner_changed_cb,
					dbus_interface = 'org.freedesktop.DBus', 
					signal_name = 'NameOwnerChanged')
		
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		self.bus_interface.ListNames (reply_handler = self.list_names_handler, 
					      error_handler = self.list_names_error_handler)
	
	def dispose(self):
		if self.skype_in:
			self.skype_in.remove_from_connection()
			self.skype_in = None
		self.owner_changed_signal.remove()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def list_names_handler(self, names):
		if skypebus in names:
			self.init_skype_objects()
		else:
			self.items_unread = 0
			self.skype = None
		self.set_menu_buttons()
		self.update_badge()
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == skypebus:
			if new_owner:
				self.init_skype_objects()
			else:
				self.items_unread = 0
				self.skype = None
			self.set_menu_buttons()
			self.update_badge()
	
	def init_skype_objects(self):
		obj = self.bus.get_object (skypebus, skypepath)
		self.skype = dbus.Interface (obj, skypeitem)
		while self.skype.Invoke("NAME dockmanager_helper") == "CONNSTATUS OFFLINE":
			time.sleep(1)
		self.skype.Invoke("PROTOCOL 5")
	
	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
		if not self.skype:
			return
		
		self.add_menu_item("Show/Hide", "skype", "Skype")
		self.add_menu_item("1. Online", STATUS_ICON_ONLINE, "Status")
		self.add_menu_item("2. Away", STATUS_ICON_AWAY, "Status")
		self.add_menu_item("3. DND", STATUS_ICON_DND, "Status")
		self.add_menu_item("4. NA", STATUS_ICON_NA, "Status")
		self.add_menu_item("5. Skypeme", STATUS_ICON_SKYPEME, "Status")
		self.add_menu_item("6. Invisible", STATUS_ICON_INVISIBLE, "Status")
		self.add_menu_item("7. Offline", STATUS_ICON_OFFLINE, "Status")
	
	def menu_pressed(self, menu_id):
		if not self.skype:
			return
		
		menu_id = self.id_map[menu_id]
		
		if menu_id == "Show/Hide":
			self.show_hide()
		elif menu_id == "2. Away":
			self.skype.Invoke("SET USERSTATUS AWAY")
		elif menu_id == "1. Online":
			self.skype.Invoke("SET USERSTATUS Online")
		elif menu_id == "5. Skypeme":
			self.skype.Invoke("SET USERSTATUS SKYPEME")
		elif menu_id == "7. Offline":
			self.skype.Invoke("SET USERSTATUS OFFLINE")
		elif menu_id == "3. DND":
			self.skype.Invoke("SET USERSTATUS DND")
		elif menu_id == "4. NA":
			self.skype.Invoke("SET USERSTATUS NA")
		elif menu_id == "6. Invisible":
			self.skype.Invoke("SET USERSTATUS INVISIBLE")
		elif menu_id == "8. Options":
			self.skype.Invoke("OPEN OPTIONS general")
	
	def update_badge(self):
		if not self.skype:
			self.reset_badge()
			return
		
		if int(self.items_unread) > 0:
			self.set_badge('%s' % self.items_unread)
			self.set_tooltip('Skype - %s messages/calls' % self.items_unread)
		else:
			self.reset_tooltip()
			self.reset_badge()
			self.items_unread = 0
	
	def show_hide(self):
		state = self.skype.Invoke("GET WINDOWSTATE")
		if state == "WINDOWSTATE NORMAL":
			self.skype.Invoke("SET WINDOWSTATE HIDDEN")
		elif state == "WINDOWSTATE HIDDEN":
			self.skype.Invoke("SET WINDOWSTATE NORMAL")
	
	def check_status(self):
		if not self.skype:
			return
		notify(self.skype.Invoke("GET USERSTATUS"))
	
	def notify_cb(self, cmd):
		p = cmd.split()
		if (p[0] == "CALL"):
			if (p[2] == "STATUS"):
				if p[3] == "RINGING":
					self.set_attention()
				elif p[3] == "MISSED":
					self.items_unread = self.items_unread + 1
					self.update_badge()
			elif (p[2] == "SEEN") and (p[3] =="TRUE"):
				self.items_unread = self.items_unread - 1
				self.update_badge()
		elif p[0] == "CHATMESSAGE":
			if p[3] == "RECEIVED":
				self.set_attention()
				self.items_unread = self.items_unread + 1
				self.update_badge()
			elif p[3] == "READ":
				self.items_unread = self.items_unread - 1
				self.update_badge()
		elif p[0] == "USERSTATUS":
			if p[1] == "ONLINE":
				self.set_icon(STATUS_ICON_ONLINE)
			elif p[1] == "AWAY":
				self.set_icon(STATUS_ICON_AWAY)
			elif p[1] == "SKYPEME":
				self.set_icon(STATUS_ICON_SKYPEME)
			elif p[1] == "OFFLINE":
				self.set_icon(STATUS_ICON_OFFLINE)
			elif p[1] == "DND":
				self.set_icon(STATUS_ICON_DND)
			elif p[1] == "NA":
				self.set_icon(STATUS_ICON_NA)
			elif p[1] == "INVISIBLE":
				self.set_icon(STATUS_ICON_INVISIBLE)
		elif p[0] == "CONNSTATUS":
			if p[1] == "OFFLINE":
				self.set_icon(STATUS_ICON_OFFLINE)
			elif p[1] == "CONNECTING":
				self.set_waiting()
			else:
				self.unset_waiting()


class DockySkypeSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("skype.desktop"):
			self.items[pathtoitem] = DockySkypeItem(self, pathtoitem)

dockysink = DockySkypeSink()

def cleanup ():
	dockysink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)
	
	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))
	
	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Rico Tzschichholz, Stefan Bethge, Robert Dyer
#
#  Started from liferea_badge.py
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import dbus
import dbus.glib
import glib
import sys
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	print e
	exit()

gajimbus = "org.gajim.dbus"
gajimpath = "/org/gajim/dbus/RemoteObject"
gajimiface = "org.gajim.dbus.RemoteInterface"
	
class GajimItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.timer = 0
		self.gajim = None
		
		self.bus.add_signal_receiver(self.name_owner_changed_cb,
				dbus_interface='org.freedesktop.DBus',
				signal_name='NameOwnerChanged')
		
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		
		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)

		self.bus.add_signal_receiver(self.signal_new_message, "NewMessage", gajimiface, gajimbus, gajimpath)

	def list_names_handler(self, names):
		if gajimbus in names:
			self.init_gajim_objects()
			self.update_badge()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == gajimbus:
			if new_owner:
				self.init_gajim_objects()
			else:
				self.gajim = None
				if self.timer > 0:
					gobject.source_remove (self.timer)
					self.timer = 0
				self.update_badge()
	
	def init_gajim_objects(self):
		obj = self.bus.get_object(gajimbus, gajimpath)
		self.gajim = dbus.Interface(obj, gajimiface)

		if not self.timer > 0:
			self.timer = gobject.timeout_add (5000, self.update_badge)

	def signal_new_message(self, details):
		self.update_badge()

	def update_badge(self):
		if not self.gajim:
			self.reset_badge()
			return False
		
		items_unread = self.gajim.get_unread_msgs_number()

		if int(items_unread) > 0:
			self.set_badge("%s" % items_unread)
		else:
			self.reset_badge()
		return True

class GajimSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("gajim.desktop"):
			self.items[pathtoitem] = GajimItem(self, pathtoitem)

gajimsink = GajimSink()

def cleanup ():
	gajimsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Rico Tzschichholz, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import dbus
import dbus.glib
import glib
import sys
import os
import math

try:
	from deluge.ui.client import sclient
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	print e
	exit()

ShowUploadRate = False


def bytes2ratestr(bytes):
	for factor, suffix in (
		(1024 ** 5, 'P'),
		(1024 ** 4, 'T'), 
		(1024 ** 3, 'G'), 
		(1024 ** 2, 'M'), 
		(1024 ** 1, 'K'),
		(1024 ** 0, 'B')):
		if bytes >= factor:
			break
	amount = int(bytes / factor)
	return str(amount) + suffix
	

class DelugeItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		
		self.timer = 0
		
		sclient.set_core_uri()
	
		if not self.timer > 0:
			self.timer = gobject.timeout_add (2000, self.update_badge)

	def update_badge(self):
		rate = 0
		try:
			if ShowUploadRate:
				rate = round(sclient.get_upload_rate())
			else:	
				rate = round(sclient.get_download_rate())
	
			if rate > 0:
				self.set_badge("%s" % bytes2ratestr(rate))
			else:
				self.reset_badge()
			return True
		except Exception, e:
			print e
			self.reset_badge()
			return False
	

class DelugeSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("deluge.desktop"):
			self.items[pathtoitem] = DelugeItem(self, pathtoitem)

delugesink = DelugeSink()

def cleanup ():
	delugesink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#
# Copyright (C) 2010 Felipe Morales
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#

import gobject
import glib
import dbus
import dbus.glib
import atexit
import socket, errno
from signal import signal, SIGTERM
import sys, os
import ConfigParser

try:
	from mpd import MPDClient
except ImportError, e:
	sys.stderr.write("mpd-control: python-mpd is needed\n")
	sys.exit()
try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
except ImportError, e:
	sys.stderr.write("mpd-control: python-dockmanager is needed\n")
	sys.exit()

class DockManagerMPDItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)

		self.add_menu_item("Previous", "media-skip-backward", "MPD Controls")
		self.add_menu_item("Play/Pause", "media-playback-start", "MPD Controls")
		self.add_menu_item("Next", "media-skip-forward", "MPD Controls")

		# add a timed function that updates the item tooltip
		gobject.timeout_add(1000, self.update_title)
	
	def update_title(self):
		if self.sink.client_is_usable():
			#if there's an active song
			if self.sink.mpdclient.currentsong() != {}:
				# we'll gran the info we need from it.
				songdata = self.sink.mpdclient.currentsong()
			# if there isn't, but the playlist isn't empty
			elif self.sink.mpdclient.playlistinfo() != []:
				# we'll grab the info we need from the first song in it
				# (if we toggle playback, that's what will play)
				songdata = self.sink.mpdclient.playlistinfo()[0]
			else:
				self.set_tooltip("[nothing to play]")
				return True
			
			tail = self.sink.get_tail()
			title = self.sink.get_title(songdata)
			artist = self.sink.get_artist(songdata)
			
			if artist != "":
				self.set_tooltip(artist + " - " +  title + tail)
			else:
				self.set_tooltip(title + tail)

		return True

	def menu_pressed(self, menu_id):
		if self.id_map[menu_id] == "Play/Pause":
			self.sink.handle_actions("toggle")
		elif self.id_map[menu_id] == "Next":
			self.sink.handle_actions("next")
		elif self.id_map[menu_id] == "Previous":    
			self.sink.handle_actions("prev")

class DockManagerMPDSink(DockManagerSink):
	def __init__(self):
		DockManagerSink.__init__(self)

		# initialize self.mpdclient
		self.client_setup()

		# try to grab the multimedia keys
		self.grab_mediakeys()

	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith(("sonata.desktop", "gmpc.desktop", "ario.desktop", "gbemol.desktop")):
			self.items[pathtoitem] = DockManagerMPDItem(self, pathtoitem)

	def client_setup(self):
		try:
			self.mpdclient = MPDClient()
			self.get_config()
			self.mpdclient.connect(self.host, self.port)
			if self.password not in ("", None):
				self.mpdclient.password(self.password)
		except:
			sys.stderr.write("mpd-control: couldn't setup the client\n")
			sys.exit()

	def get_config(self):
		if self.get_config_from_sonata():
			return
		elif self.get_config_from_mpdconf("~/.mpdconf"):
			return
		elif self.get_config_from_mpdconf("/etc/mpd.conf"):
			return
		self.host = "localhost"
		self.port = 6600
		self.password = None

	def get_config_from_sonata(self):
		conf = ConfigParser.ConfigParser()
		if os.path.isfile(os.path.expanduser('~/.config/sonata/sonatarc')):
			conf.read(os.path.expanduser('~/.config/sonata/sonatarc'))
			if conf.has_option('connection', 'profile_num'):
				profile_num = conf.get('connection', 'profile_num')
				self.host=(conf.get('profiles', 'hosts[' + profile_num + ']'))
				self.port=(conf.getint('profiles', 'ports[' + profile_num + ']'))
				self.password=(conf.get('profiles', 'passwords[' + profile_num + ']'))
				return True		
		return False

	def get_config_from_mpdconf(self, path):
		self.host = "localhost"
		if os.path.isfile(os.path.expanduser(path)):
			conf = file(os.path.expanduser(path), "r")
			lines = conf.readlines()
			for line in lines:
				setting = line.split()
				if setting[0] == "bind_to_address":
					self.host = setting[1].replace('"', '')
				if setting[0] == "#port" or setting[0] == "port":
					self.port = int(setting[1].replace('"', ''))
				if setting[0] == "password":
					self.password == setting[1].replace('"', '').split("@")[0]
				elif setting[0] == "#password":
					self.password == None
		return False
		
	# check if the pipe python-mpd client needs to work is broken or not
	def client_is_usable(self):
		try:
			self.mpdclient.status()
		except socket.error, e:
			if e.errno == errno.EPIPE:
				self.client_setup()
				return False
		return True
	
	# grab multimedia keys
	def grab_mediakeys(self):
		try:
			bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
			keysbus = bus.get_object("org.gnome.SettingsDaemon", "/org/gnome/SettingsDaemon/MediaKeys")
			keysbus.GrabMediaPlayerKeys("mpd-control", 0, dbus_interface="org.gnome.SettingsDaemon.MediaKeys")
			keysbus.connect_to_signal("MediaPlayerKeyPressed", self.delegate_mediakeys)
		except:
			sys.stdout.write("mpd-control: couldn't grab media keys\n")

	# activates actions from the multimedya keys
	def delegate_mediakeys(self, *mmkeys):
		for key in mmkeys:
			if key == "Play":
				self.handle_actions("toggle")
			elif key == "Stop":
				self.handle_actions("stop")
			elif key == "Next":
				self.handle_actions("next")
			elif key == "Previous":
				self.handle_actions("prev")

	# the real action handler
	def handle_actions(self, action):
		if self.client_is_usable():
			if action == "toggle":
				if self.mpdclient.playlistinfo() != []: # we have a playlist
					if self.mpdclient.currentsong() != {}: # we have a selected song
						state = self.mpdclient.status()['state'] 
						if state == "stop": # if it's stopped, play from current selected song
							self.mpdclient.play()
						elif state in ("play", "pause"): #if it's playing or paused, toggle the state
							self.mpdclient.pause()
					else:
						self.mpdclient.play() # play from playlist start
			elif action == "prev":
				self.mpdclient.previous()
			elif action == "next":
				self.mpdclient.next()
			elif action == "stop":
				self.mpdclient.stop()

	# Returns the client status as a string 
	def get_tail(self):
		if self.mpdclient.status()['state'] == 'stop':
			return " [stopped]"
		elif self.mpdclient.status()['state'] == 'pause':
			return " [paused]"
		else:
			return ""

	# Returns song title
	def get_title(self, songdata):
		if songdata.has_key("title"):
			if songdata.has_key("name"): # we can assume it's a radio or stream
				# we split the title from the info we have
				# for streams, "title" is usually of the form "artist - title"
				return songdata["title"].split(" - ")[1]
			else:
				return songdata["title"]
		else: # there's no data
			return songdata["file"] # we return the file path

	# Returns song artist
	def get_artist(self, songdata):
		if songdata.has_key("name"): # we can assume it's a radio or stream
			if songdata.has_key("title"): # we grab the artist info from the title
				return songdata["title"].split(" - ")[0]
			else:
				return ""
		elif songdata.has_key("artist"):
			return songdata["artist"]
		else: #there's no data
			return ""

sink = DockManagerMPDSink()

def cleanup():
	sink.dispose()

if __name__ == "__main__":
	mainloop = gobject.MainLoop()

	atexit.register(cleanup)
	signal(SIGTERM, lambda signum, stacj_frame: sys.exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2009-2010 Jason Smith, Rico Tzschichholz, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import glib
import dbus
import dbus.glib
import sys
import os

try:
	import gtk
	import urllib2
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink
	from dockmanager.dockmanager import RESOURCESDIR, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	print e
	exit()

enable_art_icon = True;
enable_badge_text = True;

art_icon_from_tag = True
try:
	import mutagen.mp3
	import mutagen.mp4
	from mutagen.id3 import ID3
	#from mutagen.flac import FLAC
except	ImportError, e:
	print "python-mutagen not available - art-icon from id3v2 tag turned off"
	art_icon_from_tag = False


rhythmboxbus = "org.gnome.Rhythmbox"
playerpath = "/org/gnome/Rhythmbox/Player"
playeriface = "org.gnome.Rhythmbox.Player"

shellpath = "/org/gnome/Rhythmbox/Shell"
shelliface = "org.gnome.Rhythmbox.Shell"

album_art_tmpfile = "/tmp/dockmanager_%s_rhythmbox_helper.png" % os.getenv('USERNAME')
album_art_file = "/tmp/dockmanager_%s_rhythmbox_helper.original.png" % os.getenv('USERNAME')

# 0 - none, 1- jewel, 2 - vinyl)
overlay = 2

class RhythmboxItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)

		self.player = None
		self.shell = None

		self.elapsed_secs = 0
		self.duration_secs = 0
		self.songinfo = None
		self.cover_basename = ""
		self.current_arturl = ""
		self.current_arturl_mtime = 0

		self.bus.add_signal_receiver(self.name_owner_changed_cb,
                                             dbus_interface='org.freedesktop.DBus',
                                             signal_name='NameOwnerChanged')
                                             
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		
		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)

		self.bus.add_signal_receiver(self.signal_playingChanged, "playingChanged",  playeriface, rhythmboxbus, playerpath)
		self.bus.add_signal_receiver(self.signal_elapsedChanged, "elapsedChanged",  playeriface, rhythmboxbus, playerpath)
		self.bus.add_signal_receiver(self.signal_playingUriChanged, "playingUriChanged",  playeriface, rhythmboxbus, playerpath)
	
	def list_names_handler(self, names):
		if rhythmboxbus in names:
			self.init_rhythmbox_objects()
			self.set_menu_buttons()
			self.update_text()
			self.update_badge()
			self.update_icon()
	
			
	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == rhythmboxbus:
			if new_owner:
				self.init_rhythmbox_objects()
			else:
				self.player = None
				self.shell = None
			self.set_menu_buttons()
			self.update_text()
			self.update_badge()
			self.update_icon()
	
	
	def init_rhythmbox_objects(self):
		obj = self.bus.get_object(rhythmboxbus, playerpath)
		self.player = dbus.Interface(obj, playeriface)

		obj = self.bus.get_object(rhythmboxbus, shellpath)
		self.shell = dbus.Interface(obj, shelliface)

		if self.player and self.shell:
			self.update_songinfo(self.player.getPlayingUri())

	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
				
		if not self.player:
			return
			
		self.add_menu_item("Previous", "media-skip-backward")
		if self.rhythmbox_is_playing():
			self.add_menu_item("Pause", "media-playback-pause")
		else:
			self.add_menu_item("Play", "media-playback-start")
		self.add_menu_item("Next", "media-skip-forward")
	
	def signal_playingChanged(self, state):
		self.set_menu_buttons()
		self.update_text()
		self.update_icon()

	def signal_elapsedChanged(self, value):
		self.elapsed_secs = value
		self.update_badge()

	def signal_playingUriChanged(self, newuri):
		self.update_songinfo(newuri)
		self.update_text()

	def update_songinfo(self, uri):
		if self.shell:
			try:
				song = dict(self.shell.getSongProperties(uri))
				self.duration_secs = song.get("duration")
				if self.duration_secs > 0:
					self.songinfo = '%s - %s (%i:%02i)' % (song.get("artist", "Unknown"), song.get("title", "Unknown"), song.get("duration") / 60, song.get("duration") % 60)
				else:
					self.songinfo = '%s - %s' % (song.get("artist", "Unknown"), song.get("title", "Unknown"))
				self.cover_basename = "%s - %s" % (song.get("artist"), song.get("album"))
			except dbus.DBusException, e:
				self.duration_secs = 0
				self.songinfo = None
				self.cover_basename = ""
			return
		self.duration_secs = 0
		self.songinfo = None

	def update_icon(self):
		if not self.player:
			self.current_arturl = ""
			self.reset_icon()
			return False
			
		if not enable_art_icon:
			return True
		
		if self.rhythmbox_is_playing():
			arturl = self.get_album_art_path()
			# Add overlay to cover
			if os.path.isfile(arturl):
				if self.current_arturl == arturl and self.current_arturl_mtime == os.stat(arturl).st_mtime:
					return True
				self.current_arturl = arturl
				self.current_arturl_mtime = os.stat(arturl).st_mtime
				self.set_icon(self.get_album_art_overlay_path(arturl))
			else:
				self.current_arturl = ""
				self.reset_icon()
		else:
			self.current_arturl = ""
			self.reset_icon()
		return True
		
	def get_album_art_path(self):
		if not self.player or not self.shell:
			return ""

		arturl = ""
		playinguri = self.player.getPlayingUri()
		filename = urllib2.unquote(playinguri).replace("file://", "");

		#1. Look in song folder
		#TODO need to replace some things, this is very weird
		coverdir = os.path.dirname(filename)
		covernames = ["cover.jpg", "cover.png", "Cover.jpg", "Cover.png", 
			"album.jpg", "album.png", "Album.jpg", "Album.png", 
			"albumart.jpg", "albumart.png", "Albumart.jpg", "Albumart.png", 
			"AlbumArt.jpg", "AlbumArt.png", 
			".folder.jpg", ".folder.png", ".Folder.jpg", ".Folder.png", 
			"folder.jpg", "folder.png", "Folder.jpg", "Folder.png"]
		for covername in covernames:
			arturl = os.path.join(coverdir, covername)
			if os.path.isfile(arturl):
				return arturl

		#2. Check rhythmbox dbus song properties
		arturl = str(self.shell.getSongProperties(playinguri).get("rb:coverArt-uri", ""))
		if not arturl == "":
			return urllib2.unquote(arturl).replace("file://", "");

		#3. Look for cached cover
		arturl = os.path.expanduser("~/.cache/rhythmbox/covers/%s.jpg" % self.cover_basename)

		#4. Look for image in tags
		if art_icon_from_tag:
			image_data = None
			loader = gtk.gdk.PixbufLoader()
			properties = self.shell.getSongProperties(playinguri)
			mimetype = properties["mimetype"]
			if mimetype == "application/x-id3":
				try:
					f = ID3(filename)
				except:
					print "No ID3 tags in %s" % filename
					return arturl
				apicframes = f.getall("APIC")
				if len(apicframes) >= 1:
					frame = apicframes[0]
					image_data = frame.data
					
			elif mimetype == "audio/x-aac":
				try:
					f = mutagen.mp4.MP4(filename)
				except:
					print "MP4 couldn't open %s" % filename
					return arturl
					
				if "covr" in f.tags:
					covertag = f.tags["covr"][0]
					image_data = covertag
					
			elif mimetype == "audio/x-flac":
				#f = FLAC(filename)
				print "cover from FLAC not implemented"
					
			if image_data:
				try:
					loader.write(image_data)
					loader.close()
					pixbuf = loader.get_pixbuf()
					pixbuf.scale_simple(250, 250, gtk.gdk.INTERP_NEAREST).save(album_art_file, "png", {})
					return album_art_file
				except Exception, e:
					print "problem with image_data from %s" % filename
					print e
			try:
				loader.close()
			except:
				return arturl
			
		return arturl
	
	def get_album_art_overlay_path(self, picfile):
		if overlay == 0:
			return picfile

		try:
			pb = gtk.gdk.pixbuf_new_from_file(picfile)
		except Exception, e:
			print e
			return picfile
		
		pb_result = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 250, 250)
		pb_result.fill(0x00000000)

		if overlay == 1:
			overlayfile = os.path.join(RESOURCESDIR, "albumoverlay_jewel.png")
			pb.composite(pb_result, 30, 21, 200, 200, 30, 21, 200.0/pb.get_width(), 200.0/pb.get_height(), gtk.gdk.INTERP_BILINEAR, 255)
		elif overlay == 2:
			overlayfile = os.path.join(RESOURCESDIR, "albumoverlay_vinyl.png")
			pb.composite(pb_result, 3, 26, 190, 190, 3, 26, 190.0/pb.get_width(), 190.0/pb.get_height(), gtk.gdk.INTERP_BILINEAR, 255)
		else:
			return picfile

		pb_overlay = gtk.gdk.pixbuf_new_from_file_at_size(overlayfile, 250, 250)
		pb_overlay.composite(pb_result, 0, 0, 250, 250, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
		pb_result.save(album_art_tmpfile, "png", {})
		
		return album_art_tmpfile
		
	def update_text(self):
		if not self.shell or not self.player:
			self.reset_tooltip()

		if self.rhythmbox_is_playing() and self.songinfo:
			self.set_tooltip(self.songinfo)
		else:
			self.reset_tooltip()

	def update_badge(self):
		if not self.player:
			self.reset_badge()
		
		if not enable_badge_text:
			return True
		
		if self.rhythmbox_is_playing():
			#if song length is 0 then counting up instead of down
			if self.duration_secs > 0:
				position = self.duration_secs - self.elapsed_secs
			else:
				position = self.elapsed_secs
			string = '%i:%02i' % (position / 60, position % 60)
			self.set_badge(string)
		else:
			self.reset_badge()
	
	def menu_pressed(self, menu_id):
		if self.id_map[menu_id] == "Play":
			self.rhythmbox_playPause()
		elif self.id_map[menu_id] == "Pause":
			self.rhythmbox_playPause()
		elif self.id_map[menu_id] == "Next":
			self.rhythmbox_next()
		elif self.id_map[menu_id] == "Previous":
			self.rhythmbox_prev()
		
	def rhythmbox_playPause(self):
		if self.player:
			self.player.playPause(True)
		
	def rhythmbox_next(self):
		if self.player:
			self.player.next()
		
	def rhythmbox_prev(self):
		if self.player:
			self.player.previous()
	
	def rhythmbox_is_playing(self):
		if self.player:
			try:
				return self.player.getPlaying() == 1
			except dbus.DBusException, e:
				return False		
		return False
		
class RhythmboxSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("rhythmbox.desktop"):
			self.items[pathtoitem] = RhythmboxItem(self, pathtoitem)

rhythmboxsink = RhythmboxSink()

def cleanup ():
	rhythmboxsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	while mainloop.is_running():
		mainloop.run()



#  
#  Copyright (C) 2009-2010 Jason Smith, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import gobject
import time
import glib
import dbus
import dbus.glib
import sys
import urllib
import os
import subprocess


DOCKMANAGER_BUS   = 'net.launchpad.DockManager'
DOCKMANAGER_PATH  = '/net/launchpad/DockManager'
DOCKMANAGER_IFACE = 'net.launchpad.DockManager'

DOCKITEM_IFACE    = 'net.launchpad.DockItem'

RESOURCESDIR = "/usr/share/dockmanager/data"

class DockManagerItem():
	def __init__(self, sink, path):
		self.sink = sink;
		self.path = path
		self.bus = dbus.SessionBus()
		self.id_map = {}
		self.id_info = {}
		
		try:
			obj = self.bus.get_object(DOCKMANAGER_BUS, self.path)
			self.iface = dbus.Interface(obj, DOCKITEM_IFACE)

			self.bus.add_signal_receiver(self.menu_pressed_signal, "MenuItemActivated", DOCKITEM_IFACE, DOCKMANAGER_BUS, self.path)
			if "x-docky-uses-menu-confirmation" in self.sink.capabilities:
				self.bus.add_signal_receiver(self.menu_item_confirmation_needed, "MenuItemConfirmationNeeded", DOCKITEM_IFACE, DOCKMANAGER_BUS, self.path)
		except dbus.DBusException, e:
			print "DockManagerItem(): %s" % e
			sys.exit(0)
		
	def add_menu_item(self, name, icon, group=None):
		try:
			hints = {"label":name}
			if group:
				hints["container-title"] = group
			if icon and not icon == "":
				hint = "icon-name" if not os.path.isabs(icon) else "icon-file"
				hints[hint] = icon
			menu_id = self.iface.AddMenuItem(hints)
		except dbus.DBusException, e:
			return None
		self.id_map[menu_id] = name
		return menu_id

	def add_menu_item_uri(self, uri, group, title=None):
		try:
			if "menu-item-with-uri" in self.sink.capabilities:
				menu_id = self.iface.AddMenuItem({"uri":uri, "container-title":group})
				self.id_map[menu_id] = uri
				return menu_id
		except dbus.DBusException, e:
			return None
		return add_menu_item(title or uri, None, group)

	def remove_menu_item(self, menu_id):
		if menu_id in self.id_map:
			try:
				self.iface.RemoveMenuItem(menu_id)
			except dbus.DBusException, e:
				return None
			del self.id_map[menu_id]

	def menu_pressed_signal(self, menu_id):
		if self.id_map.has_key(menu_id):
			self.menu_pressed(menu_id)
	
	def menu_pressed(self, menu_id):
		if self.id_info.has_key(menu_id):
			subprocess.Popen(['xdg-open', self.id_info[menu_id]])
	
	def menu_item_confirmation_needed(self):
		if "x-docky-uses-menu-confirmation" in self.sink.capabilities:
			for menu_id, title in self.id_map.iteritems():
				try:
					self.iface.ConfirmMenuItem(menu_id)
				except dbus.DBusException, e:
					pass
	
	def set_tooltip(self, text):
		if ("dock-item-tooltip" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"tooltip":text})
			except dbus.DBusException, e:
				pass
	
	def reset_tooltip(self):
		self.set_tooltip("")
	
	def set_badge(self, text):
		if ("dock-item-badge" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"badge":text})
			except dbus.DBusException, e:
				pass
	
	def reset_badge(self):
		self.set_badge("")
	
	def set_icon(self, icon):
		if ("dock-item-icon-file" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"icon-file":icon})
			except dbus.DBusException, e:
				pass
	
	def reset_icon(self):
		self.set_icon("")
	
	def set_attention(self):
		if ("dock-item-attention" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"attention":True})
			except dbus.DBusException, e:
				pass
	
	def unset_attention(self):
		if ("dock-item-attention" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"attention":False})
			except dbus.DBusException, e:
				pass
	
	def set_waiting(self):
		if ("dock-item-waiting" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"waiting":True})
			except dbus.DBusException, e:
				pass
	
	def unset_waiting(self):
		if ("dock-item-waiting" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"waiting":False})
			except dbus.DBusException, e:
				pass
	
	def set_progress(self, progress):
		if ("dock-item-progress" in self.sink.capabilities):
			try:
				self.iface.UpdateDockItem({"progress":progress})
			except dbus.DBusException, e:
				pass
	
	def reset_progress(self):
		self.set_progress(-1)
	
	def dispose(self):
		try:
			self.reset_tooltip()
			self.reset_badge()
			self.reset_icon()
			for menu_id in self.id_map.keys():
				self.iface.RemoveMenuItem(menu_id)
		except dbus.DBusException, e:
			return


class DockManagerSink():
	def __init__(self):
		self.bus = dbus.SessionBus()
		self.capabilities = []
		self.items = {}

		try:
			obj = self.bus.get_object(DOCKMANAGER_BUS, DOCKMANAGER_PATH)
			iface = dbus.Interface(obj, DOCKMANAGER_IFACE)

			self.capabilities = iface.GetCapabilities()

			for pathtoitem in iface.GetItems():
				self.item_added(pathtoitem)

			self.bus.add_signal_receiver(self.item_added,   "ItemAdded",   DOCKMANAGER_IFACE, DOCKMANAGER_BUS, DOCKMANAGER_PATH)
			self.bus.add_signal_receiver(self.item_removed, "ItemRemoved", DOCKMANAGER_IFACE, DOCKMANAGER_BUS, DOCKMANAGER_PATH)

			self.bus.add_signal_receiver(self.name_owner_changed_cb, dbus_interface='org.freedesktop.DBus', signal_name='NameOwnerChanged')
		except dbus.DBusException, e:
			print "DockManagerSink(): %s" % e
			sys.exit(0)

	def item_path_found(self, path, item):
		pass
	
	def item_added(self, path):
		obj = self.bus.get_object(DOCKMANAGER_BUS, path)
		item = dbus.Interface(obj, DOCKITEM_IFACE)
		self.item_path_found(path, item)

	def item_removed(self, path):
		if path in self.items:
			self.items[path].dispose()
			del self.items[path]

	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == DOCKMANAGER_BUS and not new_owner:
			print "DockManagerDBus %s is gone, quitting now..." % name
			self.shut_down()				
	
	def dispose(self):
		self.bus.remove_signal_receiver(self.item_added,   "ItemAdded",   DOCKMANAGER_IFACE, DOCKMANAGER_BUS, DOCKMANAGER_PATH)
		self.bus.remove_signal_receiver(self.item_removed, "ItemRemoved", DOCKMANAGER_IFACE, DOCKMANAGER_BUS, DOCKMANAGER_PATH)
		for path in self.items.keys():
			self.item_removed(path)
	
	def shut_down(self):
		gobject.idle_add(quit, 1)




#!/usr/bin/env python

#  
#  Copyright (C) 2010 James Hewitt, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import glib
import sys
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
	from subprocess import Popen
except ImportError, e:
	exit()

monitor_with_inotify = True
try:
	import pyinotify
except ImportError, e:
	print "pyinotify not available - not monitoring for new configurations"
	monitor_with_inotify = False

rdp_bookmark_dir = os.path.expanduser("~/.tsclient") 

class TsclientItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.file_map = {}
		self.read_files()

		if monitor_with_inotify:
			wm = pyinotify.WatchManager()
			handler = TsclientMonitor(item=self)
			notifier = GobjectNotifier(wm, default_proc_fun=handler)
			wm.add_watch(rdp_bookmark_dir, pyinotify.ALL_EVENTS)

	def read_files(self):
		files = os.listdir(rdp_bookmark_dir)
		for filename in files:
			self.add_file(filename)

	def add_file(self, name):
		if name[-4:] == ".rdp":
			self.file_map[name] = self.add_menu_item(name[:-4], "tsclient", "Bookmarks")

	def remove_file(self, name):
		if name[-4:] == ".rdp":
			self.remove_menu_item(self.file_map[name])
			del self.file_map[name]

	def menu_pressed(self, menu_id):
		if self.id_map[menu_id] != None:
			filename = os.path.join(rdp_bookmark_dir, self.id_map[menu_id])
			if os.path.isfile(filename):
				self.start_tsclient(filename)

	def start_tsclient(self, filename):
		Popen(["tsclient", "-x", filename])

if monitor_with_inotify:
	class GobjectNotifier(pyinotify.Notifier):
		"""
		This notifier uses a gobject io watch to trigger event processing.

		"""
		def __init__(self, watch_manager, default_proc_fun=None, read_freq=0, threshold=0, timeout=None):
			"""
			Initializes the gobject notifier. See the
			Notifier class for the meaning of the parameters.

			"""
			pyinotify.Notifier.__init__(self, watch_manager, default_proc_fun, read_freq, threshold, timeout)
			gobject.io_add_watch(self._fd, gobject.IO_IN|gobject.IO_PRI, self.handle_read)

		def handle_read(self, source, condition):
			"""
			When gobject tells us we can read from the fd, we proceed processing
			events. This method can be overridden for handling a notification
			differently.

			"""
			self.read_events()
			self.process_events()
			return True

	class TsclientMonitor(pyinotify.ProcessEvent):
		def my_init(self, item):
			self._item = item

		def process_IN_CREATE(self, event):
			self._item.add_file(event.name)

		def process_IN_DELETE(self, event):
			self._item.remove_file(event.name)

		def process_IN_MOVED_FROM(self, event):
			self._item.remove_file(event.name)

		def process_IN_MOVED_TO(self, event):
			self._item.add_file(event.name)

class TsclientSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith("tsclient.desktop"):
			self.items[pathtoitem] = TsclientItem(self, pathtoitem)

tscsink = TsclientSink()

def cleanup():
	tscsink.dispose()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register(cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Mike Nestor <mike@kineticoding.com>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import dbus
import glib
import sys
import os
import time
import webbrowser
	
try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from dockmanager.dockmanager import RESOURCESDIR
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

STATUS_ICON_LOVE = "stock_about"
STATUS_ICON_BAN = "gtk-cancel"
STATUS_ICON_TIRED = "gtk-jump-to-ltr"
STATUS_ICON_URL = "gtk-dnd"
STATUS_ICON_NEXT = "stock_media-next"
STATUS_ICON_PLAY = "stock_media-play"
STATUS_ICON_PAUSE = "stock_media-pause" 

TEXT_PLAY = "Play"
TEXT_PAUSE = "Pause"
TEXT_TIRED = "Tired (30 Days)"
TEXT_BAN = "Ban (Forever)"
TEXT_LOVE = "Love"
TEXT_NEXT = "Next"
TEXT_URL = "Song Info"

DOCKY_NAME = "Pithos Control"

DBUS_BUS = "net.kevinmehall.Pithos"
DBUS_OBJECT_PATH = "/net/kevinmehall/Pithos"

class DockyPithosItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.menu_items_count = 0
		self.pithos = None
		self.song_info = None
		self.song_url = None

		self.owner_changed_signal = self.bus.add_signal_receiver(
					handler_function = self.name_owner_changed_cb,
					dbus_interface = 'org.freedesktop.DBus', 
					signal_name = 'NameOwnerChanged')
		
		obj = self.bus.get_object ("org.freedesktop.DBus", "/")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		self.bus_interface.ListNames (reply_handler = self.list_names_handler, 
					      error_handler = self.list_names_error_handler)
	
		#listen for media key presses and act on them
		try:
			self.gnomeSettingsListner = self.bus.get_object('org.gnome.SettingsDaemon', '/org/gnome/SettingsDaemon/MediaKeys')
			self.gnomeSettingsListner.connect_to_signal("MediaPlayerKeyPressed", self.gnomesettings_signal_handler, dbus_interface='org.gnome.SettingsDaemon.MediaKeys')
		except:
			self.gnomeSettingsListner = None
	
	def dispose(self):
		self.owner_changed_signal.remove()
		self.clear_menu_buttons()

	def gnomesettings_signal_handler(*mmkeys):
		if not self.pithos:
			return
			
		for mmk in mmkeys:
			if mmk == "Play": #for me this key is mapped to play/pause so we default to that behaviour
				self.pithos.PlayPause()
			elif mmk == "Pause":
				self.pithos.PlayPause()
			elif mmk == "Stop":
				self.pithos.BanCurrentSong()
			elif mmk == "Next":
				self.pithos.SkipSong()
			elif mmk == "Previous":
				self.pithos.TiredCurrentSong()
				
	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def list_names_handler(self, names):
		if DBUS_BUS in names:
			self.init_pithos_objects()
		else:
			self.items_unread = 0
			self.pithos = None
		self.set_menu_buttons()
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == DBUS_BUS:
			if new_owner:
				self.init_pithos_objects()
			else:
				self.items_unread = 0
				self.pithos = None
			self.set_menu_buttons()
	
	def init_pithos_objects(self):
		obj = self.bus.get_object (DBUS_BUS, DBUS_OBJECT_PATH)
		self.pithos = dbus.Interface (obj, DBUS_BUS)
		
		self.pithos.connect_to_signal("PlayStateChanged", self.play_state_changed)
		
		self.update_text()
	
	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
			self.menu_items_count = self.menu_items_count - 1
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
		
		if not self.pithos:
			return
		
		if self.pithos_is_playing():
			self.add_menu_item(TEXT_PAUSE, STATUS_ICON_PAUSE, DOCKY_NAME)
		else:
			self.add_menu_item(TEXT_PLAY, STATUS_ICON_PLAY, DOCKY_NAME)
			
		self.add_menu_item(TEXT_NEXT, STATUS_ICON_NEXT, DOCKY_NAME)
		self.add_menu_item(TEXT_LOVE, STATUS_ICON_LOVE, DOCKY_NAME)
		self.add_menu_item(TEXT_BAN, STATUS_ICON_BAN, DOCKY_NAME)
		self.add_menu_item(TEXT_TIRED, STATUS_ICON_TIRED, DOCKY_NAME)
		self.add_menu_item(TEXT_URL, STATUS_ICON_URL, DOCKY_NAME)
		
	def menu_pressed(self, menu_id):
		if not self.pithos:
			return
		
		menu_id = self.id_map[menu_id]
		
		if menu_id == TEXT_PLAY or menu_id == TEXT_PAUSE:
			self.pithos.PlayPause()
		elif menu_id == TEXT_NEXT:
			self.pithos.SkipSong()
		elif menu_id == TEXT_LOVE:
			self.pithos.LoveCurrentSong()
		elif menu_id == TEXT_BAN:
			self.pithos.BanCurrentSong()
		elif menu_id == TEXT_TIRED:
			self.pithos.TiredCurrentSong()
		elif menu_id == TEXT_URL:
			self.song_info_url()
			
	def song_info_url(self):
		if (self.song_url != None):
			webbrowser.open(self.song_url)
			
	def song_changed(self, info):
		if (info):
			self.song_info = '%s [%s] by %s' % (info['title'], info['album'], info['artist'])
			self.song_url = info['songDetailURL']
		else:
			self.song_info = None
			self.song_url = None
	
	def pithos_is_playing(self):
		if not self.pithos:
			return False
			
		return self.pithos.IsPlaying()
						
	def play_state_changed(self, state):
		self.set_menu_buttons()
		self.update_text()
		
	def update_text(self):
		self.reset_tooltip()
		
		if not self.pithos:
			return

		self.song_changed(self.pithos.GetCurrentSong())
		
		if (self.pithos_is_playing() and self.song_info != None):
			self.set_tooltip(self.song_info)
			
class DockyPithosSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("pithos.desktop"):
			self.items[pathtoitem] = DockyPithosItem(self, pathtoitem)

dockysink = DockyPithosSink()

def cleanup ():
	dockysink.dispose ()
			
if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)
	
	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))
	
	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2009-2010 Jason Smith, Rico Tzschichholz
#                2010 Lukasz Piepiora, Robert Dyer
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import dbus
import dbus.glib
import glib
import sys
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()


pidginbus = "im.pidgin.purple.PurpleService"
pidginpath = "/im/pidgin/purple/PurpleObject"
pidginitem = "im.pidgin.purple.PurpleInterface"


class PidginDBus():
	def __init__(self):
		bus = dbus.SessionBus()
		obj = bus.get_object (pidginbus, pidginpath)
		self.iface = dbus.Interface (obj, pidginitem)
		
	def IsConnected(self):
		status = self.iface.PurpleSavedstatusGetCurrent()	
		return not self.iface.PurpleSavedstatusGetType(status) == 1

	def IsAway(self):
		status = self.iface.PurpleSavedstatusGetCurrent()	
		return not self.iface.PurpleSavedstatusGetType(status) == 5
		
	def Available(self):
		new_status = self.iface.PurpleSavedstatusNew("", 2)
		self.iface.PurpleSavedstatusActivate(new_status)
	
	def Disconnect(self):
		new_status = self.iface.PurpleSavedstatusNew("", 1)
		self.iface.PurpleSavedstatusActivate(new_status)	
	
	def Away(self):
		new_status = self.iface.PurpleSavedstatusNew("", 5)
		self.iface.PurpleSavedstatusActivate(new_status)


class PidginItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.pidgin = None
		
		self.bus.add_signal_receiver(self.name_owner_changed_cb,
				dbus_interface='org.freedesktop.DBus',
				signal_name='NameOwnerChanged')
		
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		
		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)
		
		self.bus.add_signal_receiver(self.status_changed, "AccountStatusChanged", pidginitem, pidginbus, pidginpath)
		self.bus.add_signal_receiver(self.conversation_updated, "ConversationUpdated", pidginitem, pidginbus, pidginpath)

	def list_names_handler(self, names):
		if pidginbus in names:
			self.init_pidgin_objects()
			self.set_menu_buttons()
			self.update_badge()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == pidginbus:
			if new_owner:
				self.init_pidgin_objects()
			else:
				self.pidgin = None
			self.set_menu_buttons()
			self.update_badge()
	
	def init_pidgin_objects(self):
		self.pidgin = PidginDBus()

	def status_changed(self, a, b, c):
		self.set_menu_buttons()
		self.update_badge()
	
	def conversation_updated(self, conv, type):
		self.update_badge()

	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
				
		if not self.pidgin or not self.iface:
			return

		if self.pidgin.IsConnected():
			if self.pidgin.IsAway():
				self.add_menu_item ("Set Away", "/usr/share/pixmaps/pidgin/status/16/away.png")
			else:
				self.add_menu_item ("Set Available", "/usr/share/pixmaps/pidgin/status/16/available.png")
			self.add_menu_item ("Disconnect", "/usr/share/pixmaps/pidgin/status/16/offline.png")
		else:
			self.add_menu_item ("Connect", "/usr/share/pixmaps/pidgin/status/16/available.png")
		
	def update_badge(self):
		if not self.pidgin:
			self.reset_badge()
			return False
		
		convs = self.pidgin.iface.PurpleGetConversations()
		count = 0
		for conv in convs:
			count = count + self.pidgin.iface.PurpleConversationGetData(conv, "unseen-count")
		if count:
			self.set_badge("%s" % count)
		else:
			self.reset_badge()
		return True
	
	def menu_pressed(self, menu_id):
		menu_id = self.id_map[menu_id]
		
		if menu_id == "Disconnect":
			self.pidgin.Disconnect()
		elif menu_id == "Set Away":
			self.pidgin.Away()
		else:
			self.pidgin.Available()
		
	
class PidginSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("pidgin.desktop"):
			self.items[pathtoitem] = PidginItem(self, pathtoitem)


pidginsink = PidginSink()

def cleanup ():
	pidginsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Rico Tzschichholz, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#  Notes:
#  * Getting Things GNOME! 0.2 or higher is required for dbus support

import atexit
import gobject
import glib
import dbus
import dbus.glib
import sys
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

gtgbus = "org.GTG"
gtgpath = "/org/GTG"
gtgiface = "org.GTG"

class GTGItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.gtg = None
		
		self.bus.add_signal_receiver(self.name_owner_changed_cb,
                                             dbus_interface='org.freedesktop.DBus',
                                             signal_name='NameOwnerChanged')
                                             
		obj = self.bus.get_object ("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")
		
		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)
	
	def list_names_handler(self, names):
		if gtgbus in names:
			self.init_gtg_objects()
			self.set_menu_buttons()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)
	
	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == gtgbus:
			if new_owner:
				self.init_gtg_objects()
			else:
				self.gtg = None
			self.set_menu_buttons()
	
	def init_gtg_objects(self):
		obj = self.bus.get_object(gtgbus, gtgpath)
		self.gtg = dbus.Interface(obj, gtgiface)
		
	def clear_menu_buttons(self):
		for k in self.id_map.keys():
			self.remove_menu_item(k)
	
	def set_menu_buttons(self):
		self.clear_menu_buttons()
		
		if not self.gtg:
			return

		self.add_menu_item ("New Task", "list-add", "Task Controls")
		self.add_menu_item ("Open Tasks List", "gtg", "Task Controls")

	def menu_pressed(self, menu_id):
		if not menu_id in self.id_map:
			return	
		
		menu_id = self.id_map[menu_id]
		
		if menu_id == "New Task":
			self.gtg.open_new_task()
		elif menu_id == "Open Tasks List":
			self.gtg.show_task_browser()
		
		
class GTGSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("gtg.desktop"):
			self.items[pathtoitem] = GTGItem(self, pathtoitem)


gtgsink = GTGSink()

def cleanup ():
	gtgsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Tom Cowell, Rico Tzschichholz, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gobject
import glib
import dbus
import dbus.glib
import sys
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	print e
	exit()

emesenebus = "org.emesene.dbus"
emesenepath = "/org/emesene/dbus"
emeseneiface = "org.emesene.dbus"

class EmeseneItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.emesene = None

		self.bus.add_signal_receiver(self.name_owner_changed_cb, dbus_interface='org.freedesktop.DBus', signal_name = 'NameOwnerChanged')
		obj = self.bus.get_object("org.freedesktop.DBus", "/org/freedesktop/DBus")
		self.bus_interface = dbus.Interface(obj, "org.freedesktop.DBus")

		self.bus_interface.ListNames (reply_handler=self.list_names_handler, error_handler=self.list_names_error_handler)

		self.bus.add_signal_receiver(self.conversation_updated, "unread_messages", emeseneiface, emesenebus, emesenepath)
	
	def list_names_handler(self, names):
		if emesenebus in names:
			self.init_emesene_objects()
			self.update_badge()

	def list_names_error_handler(self, error):
		print "error getting bus names - %s" % str(error)

	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if name == emesenebus:
			if new_owner:
				self.init_emesene_objects()
			else:
				self.emesene = None
				self.update_badge()

	def init_emesene_objects(self):
		obj = self.bus.get_object(emesenebus, emesenepath)
		self.emesene = dbus.Interface(obj, emeseneiface)

	def conversation_updated(self, count):
		self.update_badge()

	def update_badge(self):
		if not self.emesene:
			self.reset_badge()
			return False

		count = self.emesene.get_message_count()
		if count > 0:
			self.set_badge("%s" % count)
		else:
			self.reset_badge()

		return True		

class EmeseneSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith ("emesene.desktop"):
			self.items[pathtoitem] = EmeseneItem(self, pathtoitem)

emesenesink = EmeseneSink()

def cleanup():
	emesenesink.dispose()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)
	
	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))
	
	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2009 Jason Smith, Robert Dyer
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import gconf
import gobject
import glib
import sys
import urllib
import os

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
except ImportError, e:
	exit()

class TerminalItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		
		client = gconf.client_get_default()
		self.terminal = client.get_string("/desktop/gnome/applications/terminal/exec")
		if self.terminal == None:
			self.terminal = "gnome-terminal"
		
		self.path = urllib.unquote(str(self.iface.Get(DOCKITEM_IFACE, "Uri", dbus_interface="org.freedesktop.DBus.Properties")[7:]))
		if not os.path.isdir (self.path):
			self.path = os.path.dirname (self.path)
		
		self.add_menu_item("Open Terminal Here", "terminal", "actions")

	def menu_pressed(self, menu_id):
		if self.id_map[menu_id] == "Open Terminal Here":
			oldDir = os.getcwd()
			os.chdir(self.path)
			os.system ('%s &' % self.terminal)
			os.chdir(oldDir)
			
class TerminalSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "Uri", dbus_interface="org.freedesktop.DBus.Properties").startswith ("file://"):
			self.items[pathtoitem] = TerminalItem(self, pathtoitem)

terminalsink = TerminalSink()

def cleanup ():
	terminalsink.dispose ()

if __name__ == "__main__":
	mainloop = gobject.MainLoop(is_running=True)

	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))

	mainloop.run()



#!/usr/bin/env python

#  
#  Copyright (C) 2010 Dan Korostelev, Rico Tzschichholz, Robert Dyer, Michal Hruby
# 
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import atexit
import dbus
import gobject

try:
	from dockmanager.dockmanager import DockManagerItem, DockManagerSink, DOCKITEM_IFACE
	from signal import signal, SIGTERM
	from sys import exit
	import urllib2
	import json
except ImportError, e:
	print e
	exit()

transmissionbus = "com.transmissionbt.Transmission"
transmissionrpcurl = "http://localhost:9091/transmission/rpc"
UPDATE_DELAY = 3000 # 3 secs


def bytes2ratestr(bytes):
	for factor, suffix in (
		(1024 ** 5, 'P'),
		(1024 ** 4, 'T'), 
		(1024 ** 3, 'G'), 
		(1024 ** 2, 'M'), 
		(1024 ** 1, 'K'),
		(1024 ** 0, 'B')):
		if bytes >= factor:
			break
	amount = int(bytes / factor)
	return str(amount) + suffix


class TransmissionItem(DockManagerItem):
	def __init__(self, sink, path):
		DockManagerItem.__init__(self, sink, path)
		self.timer = 0
		
		self.down_speed = ""
		self.up_speed = ""
		self.progress = -1
		
		self.bus.add_signal_receiver(self.name_owner_changed_cb,
		                             dbus_interface='org.freedesktop.DBus',
		                             signal_name='NameOwnerChanged',
		                             arg0=transmissionbus)
		
		self.start_polling()

	def name_owner_changed_cb(self, name, old_owner, new_owner):
		if new_owner:
			# transmission started, resume polling
			self.start_polling()
		else:
			# transmission stopped, stop polling
			self.stop_polling()

	def start_polling(self):
		if not self.timer > 0:
			self.timer = gobject.timeout_add(UPDATE_DELAY, self.refresh_item)
			self.refresh_item()

	def stop_polling(self):
		if self.timer > 0:
			gobject.source_remove(self.timer)
			self.timer = 0
		self.reset_badge()
		self.reset_progress()
		self.reset_tooltip()

	def refresh_item(self):
		try:
			self.update_stats()
			
			self.set_progress(self.progress)
			
			if len(self.down_speed) > 0:
				self.set_badge(self.down_speed)
			else:
				self.reset_badge()
			
			if len(self.down_speed) > 0 and len(self.up_speed) > 0:
				self.set_tooltip("Transmission - " + self.down_speed + "B/s down " + self.up_speed + "B/s up")
			elif len(self.down_speed) > 0:
				self.set_tooltip("Transmission - " + self.down_speed + "B/s down")
			elif len(self.up_speed) > 0:
				self.set_tooltip("Transmission - " + self.up_speed + "B/s up")
		except Exception as e:
			return False
		return True
	
	def get_transmission_data(self, request):
		result = None
		
		for i in range(2): # first try can be getting session id
			try:
				response = urllib2.urlopen(request)
				result = json.load(response)
				break # if no error, we don't need second try
			except urllib2.HTTPError, e:
				result = None
				if e.code == 409 and 'X-Transmission-Session-Id' in e.headers:
					request.add_header('X-Transmission-Session-Id', e.headers['X-Transmission-Session-Id'])
		
		return result
	
	def update_stats(self):
		try:
			# update the speeds
			request = urllib2.Request(transmissionrpcurl)
			req_info = {'method':'session-stats'}
			request.add_data(json.dumps(req_info))
			result = self.get_transmission_data(request)
			
			self.down_speed = ""
			self.up_speed = ""
			if result:
				speed = result['arguments']['downloadSpeed']
				if speed:
					self.down_speed = bytes2ratestr(speed)
				speed = result['arguments']['uploadSpeed']
				if speed:
					self.up_speed = bytes2ratestr(speed)
			
			# update the progress
			request = urllib2.Request(transmissionrpcurl)
			req_info = {'method':'torrent-get', 'arguments':{'fields':['percentDone', 'status']}}
			request.add_data(json.dumps(req_info))
			result = self.get_transmission_data(request)
			
			self.progress = -1
			if result:
				percents = result['arguments']['torrents']
				total_percent = num_download = 0
				TR_STATUS_DOWNLOADING = 4
				
				for torrent in percents:
					if torrent['status'] & TR_STATUS_DOWNLOADING != 0:
						num_download += 1
						total_percent += torrent['percentDone']
				
				if (num_download > 0):
					self.progress = int(total_percent / num_download * 100)
		except Exception as e:
			self.stop_polling()


class TransmissionSink(DockManagerSink):
	def item_path_found(self, pathtoitem, item):
		if item.Get(DOCKITEM_IFACE, "DesktopFile", dbus_interface="org.freedesktop.DBus.Properties").endswith("transmission.desktop"):
			self.items[pathtoitem] = TransmissionItem(self, pathtoitem)

transmissionsink = TransmissionSink()

def cleanup ():
	transmissionsink.dispose ()

if __name__ == '__main__':
	mainloop = gobject.MainLoop(is_running=True)
	
	atexit.register (cleanup)
	signal(SIGTERM, lambda signum, stack_frame: exit(1))
	
	mainloop.run()
